"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Runtime = void 0;
const internal_1 = require("../internal");
class Runtime {
    constructor(agileInstance) {
        this.currentJob = null;
        this.jobQueue = [];
        this.jobsToRerender = [];
        this.notReadyJobsToRerender = new Set();
        this.isPerformingJobs = false;
        this.agileInstance = () => agileInstance;
    }
    ingest(job, config = {}) {
        config = internal_1.defineConfig(config, {
            perform: !this.isPerformingJobs,
        });
        this.jobQueue.push(job);
        internal_1.Agile.logger.if
            .tag(['runtime'])
            .info(internal_1.LogCodeManager.getLog('16:01:00', [job._key]), job);
        if (config.perform) {
            const performJob = this.jobQueue.shift();
            if (performJob)
                this.perform(performJob);
        }
    }
    perform(job) {
        this.isPerformingJobs = true;
        this.currentJob = job;
        job.observer.perform(job);
        job.performed = true;
        job.observer.dependents.forEach((observer) => observer.ingest());
        if (job.rerender)
            this.jobsToRerender.push(job);
        this.currentJob = null;
        internal_1.Agile.logger.if
            .tag(['runtime'])
            .info(internal_1.LogCodeManager.getLog('16:01:01', [job._key]), job);
        if (this.jobQueue.length > 0) {
            const performJob = this.jobQueue.shift();
            if (performJob)
                this.perform(performJob);
        }
        else {
            this.isPerformingJobs = false;
            if (this.jobsToRerender.length > 0) {
                setTimeout(() => {
                    this.updateSubscribers();
                });
            }
        }
    }
    updateSubscribers() {
        const jobsToRerender = this.jobsToRerender.concat(Array.from(this.notReadyJobsToRerender));
        this.notReadyJobsToRerender = new Set();
        this.jobsToRerender = [];
        if (!this.agileInstance().hasIntegration() || jobsToRerender.length <= 0)
            return false;
        const subscriptionContainerToUpdate = this.extractToUpdateSubscriptionContainer(jobsToRerender);
        if (subscriptionContainerToUpdate.length <= 0)
            return false;
        this.updateSubscriptionContainer(subscriptionContainerToUpdate);
        return true;
    }
    extractToUpdateSubscriptionContainer(jobs) {
        const subscriptionsToUpdate = new Set();
        jobs.forEach((job) => {
            job.subscriptionContainersToUpdate.forEach((subscriptionContainer) => {
                let updateSubscriptionContainer = true;
                if (!subscriptionContainer.ready) {
                    if (!job.config.maxTriesToUpdate ||
                        job.timesTriedToUpdateCount < job.config.maxTriesToUpdate) {
                        job.timesTriedToUpdateCount++;
                        this.notReadyJobsToRerender.add(job);
                        internal_1.LogCodeManager.log('16:02:00', [subscriptionContainer.key], subscriptionContainer);
                    }
                    else {
                        internal_1.LogCodeManager.log('16:02:01', [job.config.maxTriesToUpdate], subscriptionContainer);
                    }
                    return;
                }
                updateSubscriptionContainer =
                    updateSubscriptionContainer &&
                        this.handleSelectors(subscriptionContainer, job);
                if (updateSubscriptionContainer) {
                    subscriptionContainer.updatedSubscribers.add(job.observer);
                    subscriptionsToUpdate.add(subscriptionContainer);
                }
                job.subscriptionContainersToUpdate.delete(subscriptionContainer);
            });
        });
        return Array.from(subscriptionsToUpdate);
    }
    updateSubscriptionContainer(subscriptionsToUpdate) {
        subscriptionsToUpdate.forEach((subscriptionContainer) => {
            if (subscriptionContainer instanceof internal_1.CallbackSubscriptionContainer)
                subscriptionContainer.callback();
            if (subscriptionContainer instanceof internal_1.ComponentSubscriptionContainer)
                this.agileInstance().integrations.update(subscriptionContainer.component, this.getUpdatedObserverValues(subscriptionContainer));
            subscriptionContainer.updatedSubscribers.clear();
        });
        internal_1.Agile.logger.if
            .tag(['runtime'])
            .info(internal_1.LogCodeManager.getLog('16:01:02'), subscriptionsToUpdate);
    }
    getUpdatedObserverValues(subscriptionContainer) {
        var _a;
        const props = {};
        for (const observer of subscriptionContainer.updatedSubscribers) {
            const key = (_a = subscriptionContainer.subscriberKeysWeakMap.get(observer)) !== null && _a !== void 0 ? _a : observer.key;
            if (key != null)
                props[key] = observer.value;
        }
        return props;
    }
    handleSelectors(subscriptionContainer, job) {
        var _a;
        const selectorMethods = (_a = subscriptionContainer.selectorsWeakMap.get(job.observer)) === null || _a === void 0 ? void 0 : _a.methods;
        if (selectorMethods == null)
            return true;
        const previousValue = job.observer.previousValue;
        const newValue = job.observer.value;
        for (const selectorMethod of selectorMethods) {
            if (internal_1.notEqual(selectorMethod(newValue), selectorMethod(previousValue)))
                return true;
        }
        return false;
    }
}
exports.Runtime = Runtime;
